import os
from cronohub import source_plugin
from pathlib import Path

class SourcePlugin(source_plugin.CronohubSourcePlugin):
    def __init__(self):
        print('initialising local source plugin')

    def validate(self):
        print('validating')
        if 'CRONOHUB_LOCAL_SOURCE' not in os.environ:
            print('Please set CRONOHUB_LOCAL_SOURCE to a directory from which to uploda files from.')
            return False
        return True

    def help(self):
        print('Define a local folder from which to upload all files from to target.')

    def fetch(self):
        folder = os.environ['CRONOHUB_LOCAL_SOURCE']
        result = []
        l = list(Path(folder).rglob("*.*"))
        for p in l:
            result.append((os.path.basename(p), str(p)))
        return result


import argparse
import sys
from importlib import import_module
from pathlib import Path

import pkg_resources
from colored import attr, fg

parser = argparse.ArgumentParser(description='Cronohub')
parser.add_argument('-s',
                    action="store",
                    default="github",
                    type=str,
                    dest="source",
                    help="Name of the source plugin.")
parser.add_argument('-t',
                    action="store",
                    default="scp",
                    type=str,
                    dest="target",
                    help="Name of the target plugin.")
parser.add_argument('--source-help',
                    action="store",
                    default=None,
                    type=str,
                    dest="source_help",
                    help="Display help for a given source plugin.")
parser.add_argument('--target-help',
                    action="store",
                    default=None,
                    type=str,
                    dest="target_help",
                    help="Display help for a given target plugin.")
args = parser.parse_args()


def load_from_plugin_folder(t: str, name: str):
    filepath = Path.home() / '.config' / 'cronohub' / 'plugins'
    if not filepath.exists():
        return None
    return load_plugin(t, name, filepath)


def load_from_resource_folder(t: str, name: str):
    filepath = pkg_resources.resource_filename('cronohub_plugins', '.')
    return load_plugin(t, name, filepath)


def load_plugin(t: str, name: str, filepath: Path):
    path = Path(filepath) / t
    found = False
    plugin = ''
    for p in path.iterdir():
        if name in str(p):
            found = True
            plugin = p
            break

    if not found:
        return None

    return import_module("cronohub_plugins." + t + "." + name, plugin)


def load_plugin_with_fallback(t: str, name: str):
    """
    First: ~/.config/cronohub/plugins
        Return False if fails
    Second: site-packages/cronohub-plugins
        Returns False if fails
    If the second stage fails the program exists with status code 1.
    """
    p = load_from_plugin_folder(t, name)
    if p:
        return p
    else:
        return load_from_resource_folder(t, name)


def display_help(t: str):
    if t == 'source':
        plugin = load_plugin_with_fallback(t, args.source_help)
        if not plugin:
            print('plugin %s not found' % args.source_help)
            sys.exit(1)
        plugin.SourcePlugin().help()
    else:
        plugin = load_plugin_with_fallback(t, args.target_help)
        if not plugin:
            print('plugin %s not found' % args.target_help)
            sys.exit(1)
        plugin.TargetPlugin().help()
    sys.exit(0)


def main():
    """
    The main of cronohub. Initialize the two plugins, source and target.
    If either of them is missing, output an error message. If all is well,
    call the plugin's validate function to see if everything is set up for
    the plugin that it needs. If all is well again, call fetch and archive.
    :return: None
    """
    swag = """
     _______  ______    _______  __    _  _______  __   __  __   __  _______
    |       ||    _ |  |       ||  |  | ||       ||  | |  ||  | |  ||  _    |
    |       ||   | ||  |   _   ||   |_| ||   _   ||  |_|  ||  | |  || |_|   |
    |       ||   |_||_ |  | |  ||       ||  | |  ||       ||  |_|  ||       |
    |      _||    __  ||  |_|  ||  _    ||  |_|  ||       ||       ||  _   |
    |     |_ |   |  | ||       || | |   ||       ||   _   ||       || |_|   |
    |_______||___|  |_||_______||_|  |__||_______||__| |__||_______||_______|

    Beginning archiving...
    """
    print('%s %s %s' % (fg('cyan'), swag, attr('reset')))

    if args.source_help:
        display_help('source')

    if args.target_help:
        display_help('target')

    source_plugin = load_plugin_with_fallback('source', args.source)
    if not source_plugin:
        print("source plugin %s'%s'%s not found!" % (fg('red'), args.source, attr('reset')))
        sys.exit(1)
    target_plugin = load_plugin_with_fallback('target', args.target)
    if not target_plugin:
        print("target plugin %s'%s'%s not found!" % (fg('red'), args.target, attr('reset')))
        sys.exit(1)

    sp = source_plugin.SourcePlugin()
    if not sp.validate():
        sys.exit(1)

    tp = target_plugin.TargetPlugin()
    if not tp.validate():
        sys.exit(1)

    results = sp.fetch()
    tp.archive(results)

    print('All done. Good bye.')



import owncloud
import os
from cronohub import target_plugin


class TargetPlugin(target_plugin.CronohubTargetPlugin):
    def __init__(self):
        print('initialising no op target plugin')

    def validate(self):
        for v in [
            'CRONOHUB_OWNCLOUD_URL',
            'CRONOHUB_OWNCLOUD_USERNAME',
            'CRONOHUB_OWNCLOUD_PASSWORD'
        ]:
            if v not in os.environ:
                print("Please set %s environment variable." % v)
                return False
        return True

    def help(self):
        print('''
        OwnCloud location URL: CRONOHUB_OWNCLOUD_URL (http://domain.tld/owncloud)
        OwnCloud Username: CRONOHUB_OWNCLOUD_USERNAME (admin)
        OwnCloud Password: CRONOHUB_OWNCLOUD_PASSWORD (admin)
        ''')

    def archive(self, files):
        url = os.environ['CRONOHUB_OWNCLOUD_URL']
        user = os.environ['CRONOHUB_OWNCLOUD_USERNAME']
        password = os.environ['CRONOHUB_OWNCLOUD_PASSWORD']
        oc = owncloud.Client(url)
        oc.login(user, password)
        for f in files:
            print("uploading %s..." % f[0])
            oc.put_file(f[0], f[1])



import os
import time
from cronohub import source_plugin
from multiprocessing import Pool
from pathlib import Path
from typing import List
from urllib.request import urlretrieve
from github import Github, Repository


class SourcePlugin(source_plugin.CronohubSourcePlugin):
    """
    GithubPlugin: This is a CronoHub source plugin. The source
    is Github repositories for a given user. The user is determined
    through the provided token. No other authentication is required.
    Methods:
        __init__: will initialise this plugin
        validate: validates if the plugin's required settings are met.
        help: displays a helpful message of what the plugin requires.
        fetch: the brain of the plugin will fetch the user's repositories.
    """
    def __init__(self):
        print('initializing github source plugin')

    def validate(self) -> bool:
        print('validating requirements')
        if "CRONO_GITHUB_TOKEN" not in os.environ:
            print("Please set up a token by CRONO_GITHUB_TOKEN=<token>.")
            return False
        return True

    def help(self):
        print('''
        Help (github source plugin):
            - Environment Property:
                CRONO_GITHUB_TOKEN: a token with access to listing repositories for a given user.
            - File that filters the list of repositories to archive. If not present, all will be archived.
                ~/.config/cronohub/.repo_list
        ''')

    def download(self, repo_url: (str, str)) -> (str, str):
        """
        Download a single URL. This is used by `download_urls` as the function
        to map to.
        """
        url, name = repo_url
        timestr = time.strftime("%Y%m%d-%H%M%S")
        filename = "{}_{}.zip".format(name, timestr)
        target = Path.cwd() / "target" / filename
        urlretrieve(url, target)
        return target, name

    def download_urls(self, urls):
        """
        Multithreaded url downloader.
        """
        target = Path.cwd() / "target"
        if not target.exists():
            os.makedirs(str(target))
        result = []
        with Pool(5) as p:
            result.append(p.map(self.download, urls))
        return result[0]

    def get_repo_list(self) -> List[Repository.Repository]:
        """
        Gather a list of remote forks for a given user.
        Only repositories are selected for which the given user is an owner.
        This prevents the inclusion of Company based forks.

        :return: List[Repository.Repository]
            A list of remote forks for the current user.
        """
        g = Github(os.environ['CRONO_GITHUB_TOKEN'])
        repos = []
        user = g.get_user()
        for repo in user.get_repos(type="owner"):
            if not repo.fork:
                repos.append(repo)
        return repos

    def gather_archive_urls(self, repos: List[Repository.Repository]):
        """
        Gather a list of archive links for the repositories.
        """
        return list(map(lambda r: (r.get_archive_link(archive_format="zipball"), r.name), repos))

    def fetch(self):
        repo_list = Path.home() / '.config' / 'cronohub' / '.repo_list'
        only = []
        if repo_list.is_file():
            with open(str(repo_list)) as conf:
                for line in conf:
                    only.append(line.strip())

        repos = self.get_repo_list()
        if len(only) > 0:
            repos = list(filter(lambda r: r.name in only, repos))

        urls = self.gather_archive_urls(repos)
        return self.download_urls(urls)



import time
import os
from cronohub import source_plugin
from gitlab import Gitlab, v4
from pathlib import Path
from typing import List

class SourcePlugin(source_plugin.CronohubSourcePlugin):
    gl = None
    cfg_file = None

    def __init__(self):
        print("initialising gitlab archiver plugin")

    def help(self):
        print('''Help:
            Gitlab config file under ~/.config/cronohub/gitlab.cfg
            [Optional]: Environment Property: CRONOHUB_GITLAB_ID
        ''')

    def validate(self):
        self.cfg_file = Path.home() / '.config' / 'cronohub' / 'source_gitlab.ini'
        if not self.cfg_file.exists():
            print('missing configuration file from ~/.config/cronohub/source_gitlab.ini')
            return False
        return True

    def get_repo_list(self):
        projects = self.gl.projects.list(owned=True)
        return projects

    def fetch(self):
        paths = []
        gitlab_id = 'gitlab'
        if 'CRONOHUB_GITLAB_ID' in os.environ:
            gitlab_id = os.environ['CRONOHUB_GITLAB_ID']

        target = Path.cwd() / 'target'
        if not target.exists():
            os.makedirs(str(target))

        self.gl = Gitlab.from_config(gitlab_id=gitlab_id, config_files=[self.cfg_file])
        projects = self.get_repo_list()
        p: v4.objects.Project
        for p in projects:
            # Create the export
            export = p.exports.create({})

            # Wait for the 'finished' status
            export.refresh()
            while export.export_status != 'finished':
                time.sleep(1)
                export.refresh()

            timestr = time.strftime("%Y%m%d-%H%M%S")
            filename = "{}_{}.gz".format(p.name, timestr)
            t = Path.cwd() / 'target' / filename
            # (Location, Filename) tuple
            paths.append((str(t), filename))
            with open(str(t), 'wb') as f:
                export.download(streamed=True, action=f.write)
        return paths


from abc import ABCMeta, abstractmethod


class CronohubSourcePlugin(metaclass=ABCMeta):
    """
    This is the basic definition of a CronoHub plugin.
    """
    @abstractmethod
    def validate(self):
        ...

    @abstractmethod
    def help(self):
        ...

    @abstractmethod
    def fetch(self):
        """
        Returns a tuple (str, str) where there first parameter is the name of the
        archive and the second is the location as a full path. Exp:
        ("my-project-12345", "/home/user/projects/my-project/my-project.tar.gz")
        """
        ...



import os
from paramiko import SSHClient, SSHConfig
from scp import SCPClient
from cronohub import target_plugin


class TargetPlugin(target_plugin.CronohubTargetPlugin):
    def __init__(self):
        print('initialising SCP target plugin')

    def validate(self):
        print('validating')
        if "CRONOHUB_SCP_HOST" not in os.environ:
            print('please provide CRONOHUB_SCP_HOST environment property for archiving.')
            return False
        return True

    def help(self):
        print('''
        Help (scp target plugin):
            - Environment Property:
                CRONOHUB_SCP_HOST: host name to connect to. This can be one of two:
            #1: a hostname as defined under ~/.ssh/config file exp:
                Host pi
                    HostName 1.2.3.4
                    User username
                    Port 1234
                    IdentityFile ~/.ssh/id_rsa

                In this case the environment property would be `pi`.
            #2: an IP address or a web site address in which case it will be tried as is.
        ''')

    def archive(self, files):
        print("scp-ing: ", files)
        ssh = SSHClient()
        ssh.load_system_host_keys()
        # locate hostname in config file if not found, try connecting directly.
        hostname = os.environ['CRONOHUB_SCP_HOST']
        # TODO: This won't work on windows...
        ssh_config_file = os.path.expanduser("~/.ssh/config")
        if os.path.exists(ssh_config_file):
            config = SSHConfig()
            with open(ssh_config_file) as f:
                config.parse(f)
        settings = config.lookup(hostname)
        if settings['hostname'] != hostname:
            ssh.connect(settings['hostname'],
                        username=settings['user'],
                        key_filename=settings['identityfile'][0],
                        port=settings['port'])
        else:
            ssh.connect(hostname)

        scp = SCPClient(ssh.get_transport())
        for file in files:
            scp.put(file[0])



import boto3
import os
from cronohub import target_plugin


class TargetPlugin(target_plugin.CronohubTargetPlugin):
    def __init__(self):
        print('initialising S3 target plugin')

    def validate(self):
        print('validating')
        if "CRONOHUB_S3_BUCKETNAME" not in os.environ:
            print('please provide CRONOHUB_S3_BUCKETNAME environment property for archiving.')
            return False
        return True

    def help(self):
        print('''
        Help (s3 target plugin):
            - Environment Property:
                CRONOHUB_S3_BUCKETNAME: bucket name to use to upload the files to
            - Requires a fully configured and working AWS cli environment for BOTO
        ''')

    def archive(self, files):
        bucket = os.environ['CRONOHUB_S3_BUCKETNAME']
        s3 = boto3.client('s3')
        for file in files:
            filename = os.path.basename(file[0])
            print('uploading %s to bucket %s' % (filename, bucket))
            s3.upload_file(file, bucket, filename)
            print('uploading complete')



from cronohub import target_plugin


class TargetPlugin(target_plugin.CronohubTargetPlugin):
    def __init__(self):
        print('initialising no op target plugin')

    def validate(self):
        print('validating')
        return True

    def help(self):
        print('No-Op means no operation.')

    def archive(self, files):
        print('archiving: ', files)



"""Packaging settings."""


from setuptools import Command, find_packages, setup
from cronohub import __version__


with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name='cronohub',
    version=__version__,
    include_package_data=True,
    description='Archive your github repositories to anywhere you want.',
    long_description=long_description,
    url='https://github.com/cronohub',
    author='Gergely Brautigam',
    author_email='gergely.brautigam@gmail.com',
    license='LICENSE.txt',
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='cli,git',
    packages=find_packages(exclude=['docs', 'test*']),
    install_requires=['PyGithub', 'setuptools', 'wheel'],
    entry_points={
        'console_scripts': [
            'cronohub=cronohub.cronohub:main',
        ],
    }
)